@@ -13,7 +13,7 @@ modify entries, comments or metadata, etc. or create new po files from scratch.
 """
 
 __author__ = 'David Jean Louis <izimobil@gmail.com>'
-__version__ = '1.0.4'
+__version__ = '1.0.8'
 __all__ = ['pofile', 'POFile', 'POEntry', 'mofile', 'MOFile', 'MOEntry',
            'default_encoding', 'escape', 'unescape', 'detect_encoding', ]
 
@@ -262,13 +262,25 @@ def unescape(st):
         return m  # handles escaped double quote
     return re.sub(r'\\(\\|n|t|r|")', unescape_repl, st)
 # }}}
+# function natural_sort() {{{
+
+
+def natural_sort(lst):
+    """
+    Sort naturally the given list.
+    Credits: http://stackoverflow.com/a/4836734
+    """
+    convert = lambda text: int(text) if text.isdigit() else text.lower() 
+    alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ] 
+    return sorted(lst, key = alphanum_key)
+# }}}
 # class _BaseFile {{{
 
 
 class _BaseFile(list):
     """
     Common base class for the :class:`~polib.POFile` and :class:`~polib.MOFile`
-    classes. This class should **not** be instanciated directly.
+    classes. This class should **not** be instantiated directly.
     """
 
     def __init__(self, *args, **kwargs):
@@ -339,7 +351,7 @@ class _BaseFile(list):
 
     def __contains__(self, entry):
         """
-        Overriden ``list`` method to implement the membership test (in and
+        Overridden ``list`` method to implement the membership test (in and
         not in).
         The method considers that an entry is in the file if it finds an entry
         that has the same msgid (the test is **case sensitive**) and the same
@@ -358,7 +370,7 @@ class _BaseFile(list):
 
     def append(self, entry):
         """
-        Overriden method to check for duplicates entries, if a user tries to
+        Overridden method to check for duplicates entries, if a user tries to
         add an entry that is already in the file, the method will raise a
         ``ValueError`` exception.
 
@@ -367,13 +379,15 @@ class _BaseFile(list):
         ``entry``
             an instance of :class:`~polib._BaseEntry`.
         """
-        if self.check_for_duplicates and entry in self:
+        # check_for_duplicates may not be defined (yet) when unpickling.
+        # But if pickling, we never want to check for duplicates anyway.
+        if getattr(self, 'check_for_duplicates', False) and entry in self:
             raise ValueError('Entry "%s" already exists' % entry.msgid)
         super(_BaseFile, self).append(entry)
 
     def insert(self, index, entry):
         """
-        Overriden method to check for duplicates entries, if a user tries to
+        Overridden method to check for duplicates entries, if a user tries to
         add an entry that is already in the file, the method will raise a
         ``ValueError`` exception.
 
@@ -454,7 +468,7 @@ class _BaseFile(list):
             boolean, whether to also search in entries that are obsolete.
 
         ``msgctxt``
-            string, allows to specify a specific message context for the
+            string, allows specifying a specific message context for the
             search.
         """
         if include_obsolete_entries:
@@ -483,9 +497,11 @@ class _BaseFile(list):
             'PO-Revision-Date',
             'Last-Translator',
             'Language-Team',
+            'Language',
             'MIME-Version',
             'Content-Type',
-            'Content-Transfer-Encoding'
+            'Content-Transfer-Encoding',
+            'Plural-Forms'
         ]
         ordered_data = []
         for data in data_order:
@@ -496,7 +512,7 @@ class _BaseFile(list):
                 pass
         # the rest of the metadata will be alphabetically ordered since there
         # are no specs for this AFAIK
-        for data in sorted(metadata.keys()):
+        for data in natural_sort(metadata.keys()):
             value = metadata[data]
             ordered_data.append((data, value))
         return ordered_data
@@ -609,7 +625,9 @@ class POFile(_BaseFile):
         """
         ret, headers = '', self.header.split('\n')
         for header in headers:
-            if header[:1] in [',', ':']:
+            if not len(header):
+                ret += "#\n"
+            elif header[:1] in [',', ':']:
                 ret += '#%s\n' % header
             else:
                 ret += '# %s\n' % header
@@ -639,7 +657,7 @@ class POFile(_BaseFile):
         if total == 0:
             return 100
         translated = len(self.translated_entries())
-        return int((100.00 / float(total)) * translated)
+        return int(translated * 100 / float(total))
 
     def translated_entries(self):
         """
@@ -776,7 +794,7 @@ class MOFile(_BaseFile):
 class _BaseEntry(object):
     """
     Base class for :class:`~polib.POEntry` and :class:`~polib.MOEntry` classes.
-    This class should **not** be instanciated directly.
+    This class should **not** be instantiated directly.
     """
 
     def __init__(self, *args, **kwargs):
@@ -895,9 +913,8 @@ class _BaseEntry(object):
 
         ret = ['%s%s%s "%s"' % (delflag, fieldname, plural_index,
                                 escape(lines.pop(0)))]
-        for mstr in lines:
-            #import pdb; pdb.set_trace()
-            ret.append('%s"%s"' % (delflag, escape(mstr)))
+        for line in lines:
+            ret.append('%s"%s"' % (delflag, escape(line)))
         return ret
 # }}}
 # class POEntry {{{
@@ -932,6 +949,9 @@ class POEntry(_BaseEntry):
 
         ``previous_msgid_plural``
             string, the entry previous msgid_plural.
+
+        ``linenum``
+            integer, the line number of the entry
         """
         _BaseEntry.__init__(self, *args, **kwargs)
         self.comment = kwargs.get('comment', '')
@@ -941,17 +961,18 @@ class POEntry(_BaseEntry):
         self.previous_msgctxt = kwargs.get('previous_msgctxt', None)
         self.previous_msgid = kwargs.get('previous_msgid', None)
         self.previous_msgid_plural = kwargs.get('previous_msgid_plural', None)
+        self.linenum = kwargs.get('linenum', None)
 
     def __unicode__(self, wrapwidth=78):
         """
         Returns the unicode representation of the entry.
         """
-        if self.obsolete:
-            return _BaseEntry.__unicode__(self, wrapwidth)
-
         ret = []
         # comments first, if any (with text wrapping as xgettext does)
-        comments = [('comment', '#. '), ('tcomment', '# ')]
+        if self.obsolete:
+            comments = [('tcomment', '# ')]
+        else:
+            comments = [('comment', '#. '), ('tcomment', '# ')]
         for c in comments:
             val = getattr(self, c[0])
             if val:
@@ -968,7 +989,7 @@ class POEntry(_BaseEntry):
                         ret.append('%s%s' % (c[1], comment))
 
         # occurrences (with text wrapping as xgettext does)
-        if self.occurrences:
+        if not self.obsolete and self.occurrences:
             filelist = []
             for fpath, lineno in self.occurrences:
                 if lineno:
@@ -998,17 +1019,17 @@ class POEntry(_BaseEntry):
         # previous context and previous msgid/msgid_plural
         fields = ['previous_msgctxt', 'previous_msgid',
                   'previous_msgid_plural']
+        if self.obsolete:
+            prefix = "#~| "
+        else:
+            prefix = "#| "
         for f in fields:
             val = getattr(self, f)
             if val:
-                ret += self._str_field(f, "#| ", "", val, wrapwidth)
+                ret += self._str_field(f, prefix, "", val, wrapwidth)
 
         ret.append(_BaseEntry.__unicode__(self, wrapwidth))
         ret = u('\n').join(ret)
-
-        assert isinstance(ret, text_type)
-        #if type(ret) != types.UnicodeType:
-        #    return unicode(ret, self.encoding)
         return ret
 
     def __cmp__(self, other):
@@ -1042,6 +1063,17 @@ class POEntry(_BaseEntry):
                     return 1
                 else:
                     return -1
+        # Compare msgid_plural if set
+        if self.msgid_plural:
+            if not other.msgid_plural:
+                return 1
+            for pos in self.msgid_plural:
+                if pos not in other.msgid_plural:
+                    return 1
+                if self.msgid_plural[pos] > other.msgid_plural[pos]:
+                    return 1
+                if self.msgid_plural[pos] < other.msgid_plural[pos]:
+                    return -1
         # Finally: Compare message ID
         if self.msgid > other.msgid:
             return 1
@@ -1193,7 +1225,8 @@ class _POFileParser(object):
             check_for_duplicates=kwargs.get('check_for_duplicates', False)
         )
         self.transitions = {}
-        self.current_entry = POEntry()
+        self.current_line = 0
+        self.current_entry = POEntry(linenum=self.current_line)
         self.current_state = 'st'
         self.current_token = None
         # two memo flags used in handlers
@@ -1205,7 +1238,7 @@ class _POFileParser(object):
         #     * HE: Header
         #     * TC: a translation comment
         #     * GC: a generated comment
-        #     * OC: a file/line occurence
+        #     * OC: a file/line occurrence
         #     * FL: a flags line
         #     * CT: a message context
         #     * PC: a previous msgctxt
@@ -1242,7 +1275,6 @@ class _POFileParser(object):
         Run the state machine, parse the file line by line and call process()
         with the current matched symbol.
         """
-        i = 0
 
         keywords = {
             'msgctxt': 'ct',
@@ -1255,9 +1287,9 @@ class _POFileParser(object):
             'msgid': 'pm',
             'msgctxt': 'pc',
         }
-
+        tokens = []
         for line in self.fhandle:
-            i += 1
+            self.current_line += 1
             line = line.strip()
             if line == '':
                 continue
@@ -1283,54 +1315,53 @@ class _POFileParser(object):
                 if re.search(r'([^\\]|^)"', line[1:-1]):
                     raise IOError('Syntax error in po file %s (line %s): '
                                   'unescaped double quote found' %
-                                  (self.instance.fpath, i))
+                                  (self.instance.fpath, self.current_line))
                 self.current_token = line
-                self.process(keywords[tokens[0]], i)
+                self.process(keywords[tokens[0]])
                 continue
 
             self.current_token = line
-            is_real_entry = False
 
             if tokens[0] == '#:':
                 if nb_tokens <= 1:
                     continue
                 # we are on a occurrences line
-                self.process('oc', i)
+                self.process('oc')
 
             elif line[:1] == '"':
                 # we are on a continuation line
                 if re.search(r'([^\\]|^)"', line[1:-1]):
                     raise IOError('Syntax error in po file %s (line %s): '
                                   'unescaped double quote found' %
-                                  (self.instance.fpath, i))
-                self.process('mc', i)
+                                  (self.instance.fpath, self.current_line))
+                self.process('mc')
 
             elif line[:7] == 'msgstr[':
                 # we are on a msgstr plural
-                self.process('mx', i)
+                self.process('mx')
 
             elif tokens[0] == '#,':
                 if nb_tokens <= 1:
                     continue
                 # we are on a flags line
-                self.process('fl', i)
+                self.process('fl')
 
             elif tokens[0] == '#' or tokens[0].startswith('##'):
                 if line == '#':
                     line += ' '
                 # we are on a translator comment line
-                self.process('tc', i)
+                self.process('tc')
 
             elif tokens[0] == '#.':
                 if nb_tokens <= 1:
                     continue
                 # we are on a generated comment line
-                self.process('gc', i)
+                self.process('gc')
 
             elif tokens[0] == '#|':
                 if nb_tokens <= 1:
                     raise IOError('Syntax error in po file %s (line %s)' %
-                                  (self.instance.fpath, i))
+                                  (self.instance.fpath, self.current_line))
 
                 # Remove the marker and any whitespace right after that.
                 line = line[2:].lstrip()
@@ -1338,33 +1369,35 @@ class _POFileParser(object):
 
                 if tokens[1].startswith('"'):
                     # Continuation of previous metadata.
-                    self.process('mc', i)
+                    self.process('mc')
                     continue
 
                 if nb_tokens == 2:
                     # Invalid continuation line.
                     raise IOError('Syntax error in po file %s (line %s): '
                                   'invalid continuation line' %
-                                  (self.instance.fpath, i))
+                                  (self.instance.fpath, self.current_line))
 
                 # we are on a "previous translation" comment line,
                 if tokens[1] not in prev_keywords:
                     # Unknown keyword in previous translation comment.
                     raise IOError('Syntax error in po file %s (line %s): '
                                   'unknown keyword %s' %
-                                  (self.instance.fpath, i, tokens[1]))
+                                  (self.instance.fpath, self.current_line,
+                                   tokens[1]))
 
                 # Remove the keyword and any whitespace
                 # between it and the starting quote.
                 line = line[len(tokens[1]):].lstrip()
                 self.current_token = line
-                self.process(prev_keywords[tokens[1]], i)
+                self.process(prev_keywords[tokens[1]])
 
             else:
                 raise IOError('Syntax error in po file %s (line %s)' %
-                              (self.instance.fpath, i))
+                              (self.instance.fpath, self.current_line))
 
-        if self.current_entry and not tokens[0].startswith('#'):
+        if self.current_entry and len(tokens) > 0 and \
+           not tokens[0].startswith('#'):
             # since entries are added when another entry is found, we must add
             # the last entry here (only if there are lines). Trailing comments
             # are ignored
@@ -1409,7 +1442,7 @@ class _POFileParser(object):
             action = getattr(self, 'handle_%s' % next_state)
             self.transitions[(symbol, state)] = (action, next_state)
 
-    def process(self, symbol, linenum):
+    def process(self, symbol):
         """
         Process the transition corresponding to the current state and the
         symbol provided.
@@ -1427,7 +1460,8 @@ class _POFileParser(object):
             if action():
                 self.current_state = state
         except Exception:
-            raise IOError('Syntax error in po file (line %s)' % linenum)
+            raise IOError('Syntax error in po file (line %s)' %
+                          self.current_line)
 
     # state handlers
 
@@ -1442,7 +1476,7 @@ class _POFileParser(object):
         """Handle a translator comment."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         if self.current_entry.tcomment != '':
             self.current_entry.tcomment += '\n'
         tcomment = self.current_token.lstrip('#')
@@ -1455,22 +1489,22 @@ class _POFileParser(object):
         """Handle a generated comment."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         if self.current_entry.comment != '':
             self.current_entry.comment += '\n'
         self.current_entry.comment += self.current_token[3:]
         return True
 
     def handle_oc(self):
-        """Handle a file:num occurence."""
+        """Handle a file:num occurrence."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         occurrences = self.current_token[3:].split()
         for occurrence in occurrences:
             if occurrence != '':
                 try:
-                    fil, line = occurrence.split(':')
+                    fil, line = occurrence.rsplit(':', 1)
                     if not line.isdigit():
                         fil = fil + line
                         line = ''
@@ -1483,7 +1517,7 @@ class _POFileParser(object):
         """Handle a flags line."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         self.current_entry.flags += [c.strip() for c in
                                      self.current_token[3:].split(',')]
         return True
@@ -1492,7 +1526,7 @@ class _POFileParser(object):
         """Handle a previous msgid_plural line."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         self.current_entry.previous_msgid_plural = \
             unescape(self.current_token[1:-1])
         return True
@@ -1501,7 +1535,7 @@ class _POFileParser(object):
         """Handle a previous msgid line."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         self.current_entry.previous_msgid = \
             unescape(self.current_token[1:-1])
         return True
@@ -1510,7 +1544,7 @@ class _POFileParser(object):
         """Handle a previous msgctxt line."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         self.current_entry.previous_msgctxt = \
             unescape(self.current_token[1:-1])
         return True
@@ -1519,7 +1553,7 @@ class _POFileParser(object):
         """Handle a msgctxt."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         self.current_entry.msgctxt = unescape(self.current_token[1:-1])
         return True
 
@@ -1527,7 +1561,7 @@ class _POFileParser(object):
         """Handle a msgid."""
         if self.current_state in ['mc', 'ms', 'mx']:
             self.instance.append(self.current_entry)
-            self.current_entry = POEntry()
+            self.current_entry = POEntry(linenum=self.current_line)
         self.current_entry.obsolete = self.entry_obsolete
         self.current_entry.msgid = unescape(self.current_token[1:-1])
         return True
@@ -1544,7 +1578,8 @@ class _POFileParser(object):
 
     def handle_mx(self):
         """Handle a msgstr plural."""
-        index, value = self.current_token[7], self.current_token[11:-1]
+        index = self.current_token[7]
+        value = self.current_token[self.current_token.find('"') + 1:-1]
         self.current_entry.msgstr_plural[int(index)] = unescape(value)
         self.msgstr_index = int(index)
         return True
@@ -1563,13 +1598,10 @@ class _POFileParser(object):
         elif self.current_state == 'mx':
             self.current_entry.msgstr_plural[self.msgstr_index] += token
         elif self.current_state == 'pp':
-            token = token[3:]
             self.current_entry.previous_msgid_plural += token
         elif self.current_state == 'pm':
-            token = token[3:]
             self.current_entry.previous_msgid += token
         elif self.current_state == 'pc':
-            token = token[3:]
             self.current_entry.previous_msgctxt += token
         # don't change the current state
         return False
